@@ -8,7 +8,9 @@ from dateutil import rrule
 from itertools import groupby
 from cached_property import cached_property
 
+from trytond.i18n import gettext
 from trytond.model import Workflow, ModelSQL, ModelView, fields, Unique
+from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval, Bool, If
 from trytond.pool import Pool
 from trytond.transaction import Transaction
@@ -99,12 +101,33 @@ class Asset(Workflow, ModelSQL, ModelView):
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
         depends=['currency_digits', 'state'],
-        required=True)
-    residual_value = fields.Numeric('Residual Value',
+        required=True,
+        help="The value of the asset when purchased.")
+    depreciated_amount = fields.Numeric("Depreciated Amount",
+        digits=(16, Eval('currency_digits', 2)),
+        domain=[
+            ('depreciated_amount', '<=', Eval('value')),
+            ],
         states={
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
-        depends=['currency_digits', 'state'],
+        depends=['currency_digits', 'value', 'state'],
+        required=True,
+        help="The amount already depreciated at the start date.")
+    depreciating_value = fields.Function(fields.Numeric(
+            "Depreciating Value",
+            digits=(16, Eval('currency_digits', 2)),
+            depends=['currency_digits'],
+            help="The value of the asset at the start date."),
+        'on_change_with_depreciating_value')
+    residual_value = fields.Numeric('Residual Value',
+        domain=[
+            ('residual_value', '<=', Eval('depreciating_value')),
+            ],
+        states={
+            'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
+            },
+        depends=['currency_digits', 'depreciating_value', 'state'],
         required=True,
         digits=(16, Eval('currency_digits', 2)))
     purchase_date = fields.Date('Purchase Date', states={
@@ -173,11 +196,8 @@ class Asset(Workflow, ModelSQL, ModelView):
         table = cls.__table__()
         cls._sql_constraints = [
             ('invoice_line_uniq', Unique(table, table.supplier_invoice_line),
-                'Supplier Invoice Line can be used only once on asset.'),
+                'account_asset.msg_asset_invoice_line_unique'),
             ]
-        cls._error_messages.update({
-                'delete_draft': 'Asset "%s" must be in draft to be deleted.',
-                })
         cls._transitions |= set((
                 ('draft', 'running'),
                 ('running', 'closed'),
@@ -228,6 +248,14 @@ class Asset(Workflow, ModelSQL, ModelView):
     def default_depreciation_method():
         return 'linear'
 
+    @classmethod
+    def default_depreciated_amount(cls):
+        return Decimal(0)
+
+    @classmethod
+    def default_residual_value(cls):
+        return Decimal(0)
+
     @staticmethod
     def default_start_date():
         return Pool().get('ir.date').today()
@@ -246,6 +274,13 @@ class Asset(Workflow, ModelSQL, ModelView):
             return journals[0].id
         return None
 
+    @fields.depends('value', 'depreciated_amount')
+    def on_change_with_depreciating_value(self, name=None):
+        if self.value is not None and self.depreciated_amount is not None:
+            return self.value - self.depreciated_amount
+        else:
+            return Decimal(0)
+
     @fields.depends('company')
     def on_change_with_currency_digits(self, name=None):
         if self.company:
@@ -321,9 +356,9 @@ class Asset(Workflow, ModelSQL, ModelView):
         return result
 
     def get_depreciated_amount(self):
-        lines = [line.accumulated_depreciation for line in self.lines
+        lines = [line.depreciation for line in self.lines
             if line.move and line.move.state == 'posted']
-        return max(lines) if lines else 0
+        return sum(lines, Decimal(0))
 
     def compute_move_dates(self):
         """
@@ -353,12 +388,10 @@ class Asset(Workflow, ModelSQL, ModelView):
         dates.append(self.end_date)
         return dates
 
-    def compute_depreciation(self, date, dates):
+    def compute_depreciation(self, amount, date, dates):
         """
         Returns the depreciation amount for an asset on a certain date.
         """
-        amount = (self.value - self.get_depreciated_amount()
-            - self.residual_value)
         if self.depreciation_method == 'linear':
             start_date = max([self.start_date
                     - relativedelta.relativedelta(days=1)]
@@ -393,14 +426,17 @@ class Asset(Workflow, ModelSQL, ModelView):
         Line = Pool().get('account.asset.line')
         amounts = {}
         dates = self.compute_move_dates()
-        amount = (self.value - self.get_depreciated_amount()
+        depreciated_amount = self.get_depreciated_amount()
+        amount = (self.depreciating_value
+            - depreciated_amount
             - self.residual_value)
         if amount <= 0:
             return amounts
-        residual_value, acc_depreciation = amount, Decimal(0)
+        residual_value, acc_depreciation = (
+            amount, depreciated_amount + self.depreciated_amount)
         asset_line = None
         for date in dates:
-            depreciation = self.compute_depreciation(date, dates)
+            depreciation = self.compute_depreciation(amount, date, dates)
             amounts[date] = asset_line = Line(
                 acquired_value=self.value,
                 depreciable_basis=amount,
@@ -408,22 +444,20 @@ class Asset(Workflow, ModelSQL, ModelView):
             if depreciation > residual_value:
                 asset_line.depreciation = residual_value
                 asset_line.accumulated_depreciation = (
-                    self.get_depreciated_amount()
-                    + acc_depreciation + residual_value)
+                    acc_depreciation + residual_value)
                 break
             else:
                 residual_value -= depreciation
                 acc_depreciation += depreciation
                 asset_line.depreciation = depreciation
-                asset_line.accumulated_depreciation = (
-                    self.get_depreciated_amount() + acc_depreciation)
+                asset_line.accumulated_depreciation = acc_depreciation
         else:
             if residual_value > 0 and asset_line is not None:
                 asset_line.depreciation += residual_value
                 asset_line.accumulated_depreciation += residual_value
         for asset_line in amounts.values():
-            asset_line.actual_value = (self.value -
-                asset_line.accumulated_depreciation)
+            asset_line.actual_value = (self.value
+                - asset_line.accumulated_depreciation)
         return amounts
 
     @classmethod
@@ -537,7 +571,7 @@ class Asset(Workflow, ModelSQL, ModelView):
             account=account_asset,
             )
         depreciation_line = MoveLine(
-            debit=self.get_depreciated_amount(),
+            debit=self.get_depreciated_amount() + self.depreciated_amount,
             credit=0,
             account=self.product.account_depreciation_used,
             )
@@ -609,18 +643,11 @@ class Asset(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        domain = []
-        _, operator, value = clause
-        if value is not None:
-            names = value.split(' - ', 1)
-            domain.append(('number', operator, value))
-            if len(names) != 1 and names[1]:
-                domain.append(('product', operator, value))
-            if operator.startswith('!') or operator.startswith('not'):
-                domain.insert(0, 'OR')
-        elif not operator.startswith('!') and not operator.startswith('not'):
-            domain.append(('id', '<', 0))
-        return domain
+        names = clause[2].split(' - ', 1)
+        res = [('number', clause[1], names[0])]
+        if len(names) != 1 and names[1]:
+            res.append(('product', clause[1], names[1]))
+        return res
 
     @classmethod
     def copy(cls, assets, default=None):
@@ -629,7 +656,6 @@ class Asset(Workflow, ModelSQL, ModelView):
         else:
             default = default.copy()
         default.setdefault('lines', [])
-        default.setdefault('update_moves', [])
         default.setdefault('number', None)
         default.setdefault('supplier_invoice_line', None)
         default.setdefault('move')
@@ -639,7 +665,9 @@ class Asset(Workflow, ModelSQL, ModelView):
     def delete(cls, assets):
         for asset in assets:
             if asset.state != 'draft':
-                cls.raise_user_error('delete_draft', asset.rec_name)
+                raise AccessError(
+                    gettext('account_asset.msg_delete_draft',
+                        asset=asset.rec_name))
         return super(Asset, cls).delete(assets)
 
 
@@ -650,20 +678,31 @@ class AssetLine(ModelSQL, ModelView):
         ondelete='CASCADE', readonly=True)
     date = fields.Date('Date', readonly=True)
     depreciation = fields.Numeric('Depreciation',
-        digits=(16, Eval('_parent_asset', {}).get('currency_digits', 2)),
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'],
         required=True, readonly=True)
-    acquired_value = fields.Numeric('Acquired Value', readonly=True)
-    depreciable_basis = fields.Numeric('Depreciable Basis', readonly=True)
-    actual_value = fields.Numeric('Actual Value', readonly=True)
+    acquired_value = fields.Numeric('Acquired Value', readonly=True,
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    depreciable_basis = fields.Numeric('Depreciable Basis', readonly=True,
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    actual_value = fields.Numeric('Actual Value', readonly=True,
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
     accumulated_depreciation = fields.Numeric(
-        'Accumulated Depreciation', readonly=True)
+        'Accumulated Depreciation', readonly=True,
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
 
     @classmethod
     def __setup__(cls):
         super(AssetLine, cls).__setup__()
         cls._order.insert(0, ('date', 'ASC'))
 
+    @fields.depends('asset', '_parent_asset.currency_digits')
+    def on_change_with_currency_digits(self, name=None):
+        if self.asset:
+            return self.asset.currency_digits
+
 
 class AssetUpdateMove(ModelSQL):
     'Asset - Update - Move'
@@ -717,8 +756,8 @@ class UpdateAssetShowDepreciation(ModelView):
     amount = fields.Numeric('Amount', readonly=True)
     date = fields.Date('Date', required=True,
         domain=[
-            ('date', '>=', Eval('latest_move_date')),
-            ('date', '<=', Eval('next_depreciation_date')),
+            ('date', '>', Eval('latest_move_date')),
+            ('date', '<', Eval('next_depreciation_date')),
             ],
         depends=['latest_move_date', 'next_depreciation_date'],
         help=('The date must be between the last update/depreciation date '
@@ -827,7 +866,7 @@ class UpdateAsset(Wizard):
         asset = Asset(Transaction().context['active_id'])
         latest_move_date = self.show_move.latest_move_date
         next_date = self.show_move.next_depreciation_date
-        if not (latest_move_date <= self.show_move.date <= next_date):
+        if not (latest_move_date < self.show_move.date < next_date):
             raise ValueError('The update move date is invalid')
         move = self.get_move(asset)
         move.lines = self.get_move_lines(asset)
@@ -903,10 +942,7 @@ class AssetDepreciationTable(CompanyReport):
             'closing_value',
             }
         for attr_name in grouped_attributes:
-            descr = cached_property(adder(attr_name))
-            setattr(Grouper, attr_name, descr)
-            if hasattr(descr, '__set_name__'):
-                descr.__set_name__(Grouper, attr_name)
+            setattr(Grouper, attr_name, cached_property(adder(attr_name)))
 
         return Grouper
 
@@ -924,15 +960,14 @@ class AssetDepreciationTable(CompanyReport):
 
             @cached_property
             def update_lines(self):
-                filter_ = lambda l: (l.account.kind == 'other'
+                filter_ = lambda l: (l.account.type.expense
                     and self.start_date < l.move.date <= self.end_date)
                 return list(filter(filter_,
                     (l for m in self.asset.update_moves for l in m.lines)))
 
             @cached_property
             def start_fixed_value(self):
-                if (self.start_date < self.asset.start_date
-                        or not self.asset_lines):
+                if self.start_date < self.asset.start_date:
                     return 0
                 value = self.asset_lines[0].acquired_value
                 date = self.asset_lines[0].date
@@ -1045,7 +1080,7 @@ class PrintDepreciationTable(Wizard):
         assets = Asset.search([
                 ('start_date', '<', self.start.end_date),
                 ('end_date', '>', self.start.start_date),
-                ('state', '!=', 'draft'),
+                ('state', '=', 'running'),
                 ])
         return action, {
             'ids': [a.id for a in assets],
